package com.winhc.task;


import cn.hutool.core.io.FileUtil;
import com.alibaba.fastjson.JSON;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.aliyuncs.exceptions.ClientException;
import com.winhc.bean.*;
import com.winhc.config.CaseConfig;
import com.winhc.kafka.produce.KafkaProduce;
import com.winhc.service.TouchService;
import com.winhc.utils.CompanyUtils;
import com.winhc.utils.DateUtil;
import com.winhc.utils.DingUtils;
import com.winhc.utils.SchemaInit;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;
import org.neo4j.driver.Driver;
import org.neo4j.driver.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;
import java.util.stream.Collectors;

/**
 * @author π
 * @date 2021年5月16日
 * @des 案件关联回流
 */
@Component
@Slf4j
@EnableScheduling
@AllArgsConstructor
public class CaseIncrementTask {

    @Autowired
    JdbcTemplate jdbcTemplate;

    @Autowired
    @Qualifier("DriverV2")
    Driver driver;

    @Autowired
    KafkaProduce kafkaProduce;

    @Autowired
    TouchService touchService;

    @Autowired
    DingUtils dingUtils;

    @Autowired
    CaseConfig caseConfig;

    //TODO 启动合并任务
    @Scheduled(cron = "00 10 13 * * ?")
    //@Scheduled(cron = "*/20 * * * * ?")
    //@Scheduled(cron = "30 07 10 21 06 ?")
    public void mergeCaseScheduled() throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        if (!CompanyUtils.isWindows()) {
            dingUtils.send("mergeCaseScheduled start !!!");
        }
        try {
            MergeParam param = initParams();
            exportIncrCase2CSV(param.getLastPartition(), driver.session(), param.getIncrPath());
            loadCSVSendKafka(param.getPathPre() + param.getIncrPath(), param.getTopic());
            startJob(param);
        } catch (Exception e) {
            log.error("mergeCaseScheduled error | message:{} | .", e.getMessage());
            if (!CompanyUtils.isWindows()) {
                dingUtils.send("mergeCaseScheduled job run error : \n" + e.getMessage() + "\n!!!!!!! ");
            }
        }
        if (!CompanyUtils.isWindows()) {
            dingUtils.send("mergeCaseScheduled end !!!");
        }
        log.info("mergeCaseScheduled end | cost:{} | !", (System.currentTimeMillis() - start));
    }

    public MergeParam initParams() {
        String bizdate = DateUtil.getDateBefore(-1);
        String currentPartition = DateUtil.getDateBefore(0).replace("-", "");
        String afterPartition = DateUtil.getDateBefore(1).replace("-", "");
        String before7DayPartition = DateUtil.getDateBefore(-14).replace("-", "");
        String lastPartition = bizdate.replace("-", "");
        String pathPre = "D:\\data\\opt\\";
        if (!CompanyUtils.isWindows()) {
            pathPre = "/data/opt/";
        }
        final String incrPath = "export/incr-case-" + lastPartition + ".csv";
        MergeParam param = MergeParam.builder()
                .flow(caseConfig.getFlow())
                .taskName(caseConfig.getTaskName())
                .topic(caseConfig.getTopic())
                .bizdate(bizdate)
                .lastPartition(lastPartition)
                .currentPartition(currentPartition)
                .afterPartition(afterPartition)
                .before7DayPartition(before7DayPartition)
                .pathPre(pathPre)
                .incrPath(incrPath)
                .build();
        log.info("show params : {} !", param.toString());
        return param;
    }

    /**
     * 导出增量变化数据
     *
     * @param date
     * @param session
     * @param INCR_NAME_PATH
     */
    private void exportIncrCase2CSV(String date, Session session, String INCR_NAME_PATH) {
        log.info("exportIncrCase2CSV start!");
        long start = System.currentTimeMillis();
        final String cql5 = " CALL apoc.export.csv.query('MATCH (p:" + CompanyUtils.getIncrPersonLabel("新增") + ")\n" +
                "RETURN p.case_id as case_id,p.component_id as component_id',\n" +
                "'" + INCR_NAME_PATH + "',\n" +
                "{batchSize:20000,parallel:false,retries:3,iterateList:true})\n" +
                "YIELD file,rows";
        log.info("cql5 : \n {} ", cql5);
        String res5 = CompanyUtils.runNeo4j(session, cql5);
        log.info("exportIncrCase2CSV | cost:{} | result:{} | end !", (System.currentTimeMillis() - start), res5);
    }

    /**
     * 发送kafka
     *
     * @param path
     * @param topic
     * @throws InterruptedException
     * @throws ExecutionException
     * @throws TimeoutException
     * @throws IOException
     */
    private void loadCSVSendKafka(String path, String topic) throws InterruptedException, ExecutionException, TimeoutException, IOException {
        log.info("loadCSVSendKafka  | start !");
        long start = System.currentTimeMillis();
        LineIterator it = FileUtils.lineIterator(FileUtil.file(path), "UTF-8");
        int i = 0;
        try {
            while (it.hasNext()) {
                String line = it.nextLine();
                ++i;
                if (i == 1) continue;
                Map<String, String> map = new HashMap<>();
                List<String> list = Arrays.asList(line.split("\",\"", -1)).stream()
                        .map(x -> x.replaceAll("\"", "")).collect(Collectors.toList());
                if (list.size() < 2) continue;
                String time = DateUtil.formatDate_YYYY_MM_DD_HH_MM_SS(new Date());
                String[] arr = list.get(0).split("_", -1);
                map.put("create_time", time);
                map.put("update_time", time);
                map.put("rowkey", arr[0]);
                map.put("tn", arr[1]);
                map.put("component_id", list.get(1));
                String message = JSON.toJSONString(map);
                kafkaProduce.produce(topic, message);
            }
        }catch (Exception e1){
            e1.printStackTrace();
            log.error(e1.getMessage());
        }finally {
            LineIterator.closeQuietly(it);
        }
        log.info("loadCSVSendKafka | size:{} | cost:{} | end !", i - 1, (System.currentTimeMillis() - start));
    }

    public void startJob(MergeParam param) throws InterruptedException, ClientException, IOException {
        log.info("startJob start !");
        long start = System.currentTimeMillis();
        DataWorksFlowJob dataWorksFlowJob = SchemaInit.getJobs().stream().filter(j -> j.getFlow().equals(param.getFlow())).findFirst().orElseThrow(NullPointerException::new);
        DataWorksFlowTask dataWorksFlowTask = dataWorksFlowJob.getTask().stream().filter(t -> t.getTaskName().equals(param.getTaskName())).findFirst().orElseThrow(NullPointerException::new);
        TaskParam build = TaskParam.builder()
                .projectName(dataWorksFlowJob.getProject())
                .bizDate(param.getBizdate())
                .flowName(dataWorksFlowJob.getFlow())
                .nodeParam(dataWorksFlowTask.toNodeParam(param))
                .build();
        CreateManualDagResponse touch = touchService.touch(build);
        if (touch == null) {
            return;
        }
        while (true) {
            Map<String, TaskFlowEnum> query = touchService.query(dataWorksFlowJob.getProject(), touch.getReturnValue());
            long count = query.values()
                    .stream()
                    .filter(e -> !(TaskFlowEnum.SUCCESS.equals(e) || TaskFlowEnum.FAILURE.equals(e)))
                    .count();

            long count1 = query.values().stream().filter(TaskFlowEnum.FAILURE::equals).count();
            if (count1 != 0) {
                if (!CompanyUtils.isWindows()) {
                    log.error("startJob job run error : \n" + "案件job失败" + "\n!!!!!!! ");
                    dingUtils.send("startJob job run error : \n" + "案件job失败" + "\n!!!!!!! ");
                    return;
                }
            }

            if (count != 0) {
                Thread.sleep(2 * 60 * 1000);
            } else {
                log.info("startJob end | cost:{} | !", (System.currentTimeMillis() - start));
                return;
            }
        }
    }


}
